@@ -252,12 +252,20 @@ extern int yydebug;
     VAR_DNSCRYPT_PROVIDER = 462,
     VAR_DNSCRYPT_SECRET_KEY = 463,
     VAR_DNSCRYPT_PROVIDER_CERT = 464,
-    VAR_IPSECMOD_ENABLED = 465,
-    VAR_IPSECMOD_HOOK = 466,
-    VAR_IPSECMOD_IGNORE_BOGUS = 467,
-    VAR_IPSECMOD_MAX_TTL = 468,
-    VAR_IPSECMOD_WHITELIST = 469,
-    VAR_IPSECMOD_STRICT = 470
+    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE = 465,
+    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS = 466,
+    VAR_DNSCRYPT_NONCE_CACHE_SIZE = 467,
+    VAR_DNSCRYPT_NONCE_CACHE_SLABS = 468,
+    VAR_IPSECMOD_ENABLED = 469,
+    VAR_IPSECMOD_HOOK = 470,
+    VAR_IPSECMOD_IGNORE_BOGUS = 471,
+    VAR_IPSECMOD_MAX_TTL = 472,
+    VAR_IPSECMOD_WHITELIST = 473,
+    VAR_IPSECMOD_STRICT = 474,
+    VAR_CACHEDB = 475,
+    VAR_CACHEDB_BACKEND = 476,
+    VAR_CACHEDB_SECRETSEED = 477,
+    VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM = 478
   };
 #endif
 /* Tokens.  */
@@ -468,12 +476,20 @@ extern int yydebug;
 #define VAR_DNSCRYPT_PROVIDER 462
 #define VAR_DNSCRYPT_SECRET_KEY 463
 #define VAR_DNSCRYPT_PROVIDER_CERT 464
-#define VAR_IPSECMOD_ENABLED 465
-#define VAR_IPSECMOD_HOOK 466
-#define VAR_IPSECMOD_IGNORE_BOGUS 467
-#define VAR_IPSECMOD_MAX_TTL 468
-#define VAR_IPSECMOD_WHITELIST 469
-#define VAR_IPSECMOD_STRICT 470
+#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE 465
+#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS 466
+#define VAR_DNSCRYPT_NONCE_CACHE_SIZE 467
+#define VAR_DNSCRYPT_NONCE_CACHE_SLABS 468
+#define VAR_IPSECMOD_ENABLED 469
+#define VAR_IPSECMOD_HOOK 470
+#define VAR_IPSECMOD_IGNORE_BOGUS 471
+#define VAR_IPSECMOD_MAX_TTL 472
+#define VAR_IPSECMOD_WHITELIST 473
+#define VAR_IPSECMOD_STRICT 474
+#define VAR_CACHEDB 475
+#define VAR_CACHEDB_BACKEND 476
+#define VAR_CACHEDB_SECRETSEED 477
+#define VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM 478
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
@@ -484,7 +500,7 @@ union YYSTYPE
 
 	char*	str;
 
-#line 488 "util/configparser.h" /* yacc.c:1909  */
+#line 504 "util/configparser.h" /* yacc.c:1909  */
 };
 
 typedef union YYSTYPE YYSTYPE;
